import json

import jsonpickle
from dorton.http.enums import HttpStatusCode

__all__ = [
    "APIResponse"
]

class APIResponse:
    def __init__(
        self,
        content,
        content_type="application/json",
        status_code=HttpStatusCode.OK,
    ) -> None:
        self.content = content
        self.content_type = content_type
        self.status_code = status_code

    def toJSON(self):
        return json.loads(jsonpickle.encode(self.content, unpicklable=False, indent=2))



import pathlib
from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name="Dorton",
    version="0.0.1.1",
    description="Python Lightweight Framework",
    author="wariored",
    author_email="cheikh@cheikhkonteye.com",
    packages=find_packages(),
    python_requires=">=3.9",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    long_description=long_description,
    long_description_content_type='text/markdown'
)



import json

import jsonpickle
from webob import Request, Response

from dorton.core.handlers.base import HttpHandler
from dorton.exceptions import RouteMethodNotFoundError, RouteNotFoundError
from dorton.http import APIResponse

DEFAULT_CONTENT_TYPE = "application/json"


class App(HttpHandler):
    def __call__(self, environ, start_response):
        request = Request(environ)
        response = self.handle_request(request)
        response.content_type = DEFAULT_CONTENT_TYPE
        return response(environ, start_response)

    def _default_response(self, response):
        response.status_code = 404
        response.json = "Not found."

    def handle_request(self, request: Request):
        """
        Fetch the corresponding route and execute the request

        :return WSGI response
        """
        response = Response()
        path = request.path

        try:
            handler, method, kwargs = self._find_handler(path)
        except RouteNotFoundError:
            self._default_response(response)
            return response

        if method and method != request.method:
            self._default_response(response)
        else:
            try:
                res = self._get_handler_response(request, handler, kwargs)
            except RouteMethodNotFoundError:
                self._default_response(response)
                return response
            
            if isinstance(res, APIResponse):
                response.json = res.toJSON()
                response.content_type = res.content_type
                response.status_code = res.status_code
            else:
                response.json = json.loads(
                    jsonpickle.encode(res, unpicklable=False, indent=2)
                )

        return response
